package com.reyun.bpu.dmp.utils;

import java.io.*;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.multipart.MultipartFile;
import software.amazon.awssdk.auth.credentials.InstanceProfileCredentialsProvider;
import software.amazon.awssdk.core.sync.RequestBody;
import software.amazon.awssdk.core.sync.ResponseTransformer;
import software.amazon.awssdk.services.s3.S3Client;
import software.amazon.awssdk.services.s3.model.GetObjectRequest;
import software.amazon.awssdk.services.s3.model.PutObjectRequest;
import java.nio.MappedByteBuffer;
import java.nio.file.Path;
import java.nio.file.Paths;

public class FileUtil {

	private static Logger log = LoggerFactory.getLogger(FileUtil.class);

	@Value("${s3.save.bucket}")
	private String bucket;
	@Value("${s3.save.path}")
	private String uploads3path;
	@Value("${local.path}")
	private String localpath;

	/**
	 *
	 * @param file 文件流
	 * @return
	 */
	public void uploadToS3(MultipartFile file) {

		try {

			//上传S3文件
			S3Client s3 = S3Client.builder().credentialsProvider(InstanceProfileCredentialsProvider.builder().build()).build();
			log.info("获取文件在EMR服务器上的路径");
			Path path = Paths.get(localpath+file.getOriginalFilename());
			log.info("文件从EMR写入S3开始");
			//1、写文件：这种方式需要用上传的文件先写到本地，在从本地上传到S3，因此换一种流的方式直接写到S3
			s3.putObject(PutObjectRequest.builder().bucket(bucket).key(uploads3path).build(), path);
			log.info("文件从EMR写入S3结束");
			//s3.putObject(PutObjectRequest.builder().bucket(BUCKET).key(UPLOADPATH).build(), RequestBody.fromByteBuffer(buffer));
		} catch (Exception e) {
		    log.error("文件写S3失败");
			e.printStackTrace();
		}
	}

	/**
	 * 保存文件，直接以multipartFile形式
	 * 文件先写EMR本地
	 * @param multipartFile
	 * @return 返回文件名
	 * @throws IOException
	 */
	public String saveFile(MultipartFile multipartFile) {
	    File file = new File(localpath);
		if (!file.exists()) {
			file.mkdirs();
		}
		try {
			FileInputStream input = (FileInputStream) multipartFile.getInputStream();
			BufferedOutputStream bos = null;
			bos = new BufferedOutputStream(new FileOutputStream(localpath + File.separator + multipartFile.getName()));
			byte[] bs = new byte[1024];
			int len;
			log.info("文件上传EMR开始");
			while ((len = input.read(bs)) != -1) {
				bos.write(bs, 0, len);
			}
			log.info("文件上传EMR结束");
			bos.flush();
			bos.close();
			uploadToS3(multipartFile);
		} catch (FileNotFoundException e) {
			log.error("文件没有找到");
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
			log.error("文件写入EMR本地失败");
		}
		return "文件写入EMR成功";
	}

	/**
	 * 文件复制
	 * @param source
	 * @param dest
	 * @throws IOException
	 */
	@SuppressWarnings("resource")
	public static void copyFileUsingFileChannels(File source, File dest) throws IOException {
		FileChannel inputChannel = null;
		FileChannel outputChannel = null;
		try {
			inputChannel = new FileInputStream(source).getChannel();
			outputChannel = new FileOutputStream(dest).getChannel();
			outputChannel.transferFrom(inputChannel, 0, inputChannel.size());
		} finally {
			inputChannel.close();
			outputChannel.close();
		}
	}

	public static String readFile(File file, String charset) {
		StringBuilder sb = new StringBuilder();
		BufferedReader br = null;
		InputStreamReader isr = null;
		try {
			FileInputStream fis = new FileInputStream(file);
			isr = new InputStreamReader(fis, charset);
			br = new BufferedReader(isr);
			while (br.ready()){
				sb.append(br.readLine());
				sb.append("\n");
			}

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				br.close();
				isr.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return sb.toString();
	}

	/**
	 * NIO way
	 * @throws Exception
	 */
	public static byte[] toByteArray(String filename) throws Exception {

		File f = new File(filename);
		if (!f.exists()) {
			log.error("文件未找到！" + filename);
			throw new IOException("文件未找到");
		}
		FileChannel channel = null;
		FileInputStream fs = null;
		try {
			fs = new FileInputStream(f);
			channel = fs.getChannel();
			ByteBuffer byteBuffer = ByteBuffer.allocate((int) channel.size());
			while ((channel.read(byteBuffer)) > 0) {
				// do nothing
				// System.out.println("reading");
			}
			return byteBuffer.array();
		} catch (IOException e) {
			throw new IOException("FILE_READING_ERROR");
		} finally {
			try {
				channel.close();
			} catch (IOException e) {
				throw new IOException("FILE_READING_ERROR");
			}
			try {
				fs.close();
			} catch (IOException e) {
				throw new IOException("FILE_READING_ERROR");
			}
		}
	}

	/**
	 * 删除目录
	 *
	 * @author fengshuonan
	 * @Date 2017/10/30 下午4:15
	 */
	public static boolean deleteDir(File dir) {
		if (dir.isDirectory()) {
			String[] children = dir.list();
			for (int i = 0; i < children.length; i++) {
				boolean success = deleteDir(new File(dir, children[i]));
				if (!success) {
					return false;
				}
			}
		}
		return dir.delete();
	}

	/**
	  * 删除单个文件
	 *
	 * @param fileName 要删除的文件的文件名
	 * @return 单个文件删除成功返回true，否则返回false
	 */
	public static boolean deleteFile(String fileName) {
		File file = new File(fileName);
		// 如果文件路径所对应的文件存在，并且是一个文件，则直接删除
		if (file.exists() && file.isFile()) {
			if (file.delete()) {
				log.info("删除单个文件" + fileName + "成功！");
				return true;
			} else {
				log.info("删除单个文件" + fileName + "失败！");
				return false;
			}
		} else {
			log.error("删除单个文件失败：" + fileName + "不存在！");
			return false;
		}
	}


	public static String getFileNameFromPath(String path){
		String fileName = path.trim().substring(path.lastIndexOf("/")+1);
		return fileName;
	}

	public static void main(String[] args) {
		String filename= "M00/00/00/wKgAmVmY_HWAU0H-AAAtToIKUp8155.doc";
		String fileNameFromPath = getFileNameFromPath(filename);
		System.out.println(fileNameFromPath);
	}


}